/**
 * 
 */
package week9;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Scanner;
import java.util.logging.ConsoleHandler;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Manages saving and loading of Students
 * @author ilir
 *
 */
public class StudentManager {
	private static final Logger logger =
			Logger.getLogger(StudentManager.class.getName());
	private static final Handler consoleHandler = new ConsoleHandler();
	
	/** A mapping of student ids to Students. */
	private Map<String, Student> students;
	
	/** Creates a new empty StudentManager. */
	public StudentManager() {
		students = new HashMap<String, Student>();
		
	}
	
	/** Creates a new StudentManager for the students whose
	 * information is stored in filePath
	 * @param filePath
	 * @throws FileNotFoundException 
	 * @throws IOException if filePath is not valid
	 * @throws ClassNotFoundException 
	 */
	
	public StudentManager(String filePath) throws IOException, ClassNotFoundException {
		students = new HashMap<>();
		
		logger.setLevel(Level.ALL);
		consoleHandler.setLevel(Level.ALL);
		logger.addHandler(consoleHandler);
		
		File file = new File(filePath);
		if(file.exists()) {
			readFromFile(file.getPath());
		} else
			file.createNewFile();
		
	}

	/** Populates the records map from he file at path filePath.
	 * @param filePath the path of data file.
	 * @throws FileNotFoundException if filePath is invalid
	 */
	 public void readFromCSVFile(String filePath)
			 			throws FileNotFoundException {
		 // FileInputStream can be used for reading raw bytes
		 // (like reading an image)
		 Scanner scanner = new Scanner(new FileInputStream(filePath));
		 String[] record;
		 Student student;
		 while(scanner.hasNextLine()) {
			 record = scanner.nextLine().split(",");
			 student = new Student(record[0].split(" "),
					 record[1],record[2]);
			 students.put(student.getID(), student);
		 }
		 scanner.close();
	 }
	 
	 /** Adds record to this StudentManager.
	  * @param record a record to be added.
	  */
	 public void add(Student record) {
		 students.put(record.getID(), record);
		 logger.log(Level.FINE, "Added new student " + record.toString());
	 }
	 
	 /** Writes the students to file at filePath.
	  * @param filePath the file to write records to
	  * @throws IOException
	  */
	 public void saveToFile(String filePath) throws IOException {
		 OutputStream file = new FileOutputStream(filePath);
		 OutputStream buffer = new BufferedOutputStream(file);
		 ObjectOutput output = new ObjectOutputStream(buffer);
		 
		 // serialize the map
		 output.writeObject(students);
		 output.close();
	 }
	 
	 @SuppressWarnings("unchecked")
	 private void readFromFile(String path) throws ClassNotFoundException {
		 try (
				 InputStream file = new FileInputStream(path);
				 InputStream buffer = new BufferedInputStream(file);
				 ObjectInput input = new ObjectInputStream(buffer);
				 ) {
			 // deserialize the Map
			 students = (Map<String,Student>)input.readObject();
		 } catch(IOException ex) {
			 logger.log(Level.SEVERE, "Cannot read from input", ex);
		 }
	 }
	 
	 @Override
	 public String toString() {
		 String result = "";
		 for(Student s : students.values()) {
			 result += s.toString() + "\n";
		 }
		 return result;
	 }
}
